// deferredLight.fx
// by Julian "Jibb" Smart
// 2009

const float4x4 matWorldViewProj;
const float4x4 matWorldView;
const float4x4 matView;

// Origin
float4 vecSkill1;
// Light colour and intensity
float4 vecSkill5;

float toneMappingFactor_var;

float fAlpha;

// Convert clip space to texture space
static const float4x4 mat2Tex = {
	0.5f, 0.f, 0.f, 0.f,
	0.f, -0.5f, 0.f, 0.f,
	0.f, 0.f, 1.0f, 0.f,
	0.5004f, 0.5004f, 0.f, 1.0f
};

// Actual Texture
texture entSkin1;
sampler SkinSampler = sampler_state {
	Texture = <entSkin1>;
	MipFilter = Linear;
};

// Colour Map
texture colourMap_bmap;
sampler ColourSampler = sampler_state {
	Texture = <colourMap_bmap>;
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = Linear;
};

// Depth Map
texture depthMap_bmap;
sampler DepthSampler = sampler_state {
	Texture = <depthMap_bmap>;
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = Linear;
};

// Normal Map
texture normalMap_bmap;
sampler NormalSampler = sampler_state {
	Texture = <normalMap_bmap>;
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = Linear;
};

// VS
void VS	(in float4 inPos		:	POSITION,
			in float2 inTex		:	TEXCOORD0,
			out float4 outPos		:	POSITION,
			out float2 outTex		:	TEXCOORD0,
			out float4 depthTex	:	TEXCOORD1,
			out float4 origin		:	TEXCOORD2,
			out float4 outPos2	:	TEXCOORD3) {
	float4x4 matOffset = mul(matWorldViewProj, mat2Tex);
	outPos = mul(inPos, matWorldViewProj);
	outPos2 = mul(inPos, matWorldView);
	depthTex = mul(inPos, matOffset);
	origin = mul(vecSkill1, matView);
	
	outTex = inTex;
}

// PS
float4 PS	(in float2 inTex		:	TEXCOORD0,
				in float4 depthTex	:	TEXCOORD1,
				in float4 inOrigin	:	TEXCOORD2,
				in float4 inPos		:	TEXCOORD3)	:	COLOR0 {
	// colour uses the light-mask multiplied by the colour of what we're lighting up
	// multiplied by the light's colour
	float4 colour = tex2D(SkinSampler, inTex) * tex2Dproj(ColourSampler, depthTex);
	colour.rgb *= vecSkill5.xyz;
	// calculate the relative position of the light-source from this pixel in view space
	float3 inPix;
	inPix.z = tex2Dproj(DepthSampler, depthTex).r;
	inPix.xy = inPos.xy * inPos.z / inPix.z;
	inPix = inOrigin.xyz - inPix;
	// calculate light intesnity based on direction...
	float lighting = dot(normalize(inPix), normalize(tex2Dproj(NormalSampler, depthTex).rgb*2-1));
	// ...and fall-off
	colour.rgb *= 10 * lighting * vecSkill5.w / (pow(inPix.x,2) + pow(inPix.y,2) + pow(inPix.z,2) + 0.001) * toneMappingFactor_var;
	colour.a *= fAlpha;
	return colour;
}

technique lights {
	pass p0 {
		AlphaBlendEnable = TRUE;
		ZWriteEnable = FALSE;
		ZFunc = ALWAYS;
		DestBlend = ONE;
		SrcBlend = SRCALPHA;
		VertexShader = compile vs_2_0 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
