// plugAndPlay.c
// by Julian "Jibb" Smart
// 2009

// TOON EFFECTS
/*
Toon shader with simple lighting. Outlines. Two variations:
	Dark (where outlines are just dark versions of colours);
	Black (outlines are black);

Try setting a toon entity's TRANSLUCENT flag and setting its alpha to zero;
it looks really cool having just the outlines visible!

In a manner similar to the improved transparency further down, parts of the
model won't show up through itself. Standard transparency betrays the hollow
nature of the models and generally looks uglier, but that's up to you.
*/

MATERIAL* mtl_toonDark = {
	effect = "
	const float4x4 matWorldViewProj;
	const float4x4 matWorld;
	const float4 vecViewPos;
	const float4 vecSunDir;
	float fAlpha;

	// Actual Texture
	texture entSkin1;
	sampler ColourSampler = sampler_state {
		Texture = <entSkin1>;
		MipFilter = Linear;
	};

	// DEPTH
	void VSD	(in float4 inPos		:	POSITION,
	out float4 outPos		:	POSITION) {
		outPos = mul(inPos, matWorldViewProj);
	}

	float4 PSD	() : COLOR0 {
		return (1.0, 1.0, 1.0, 1.0);
	}

	// COLOURS
	void VSC ( 
	in float4 inPos: POSITION, 
	in float3 inNormal: NORMAL, 
	in float2 inTex: TEXCOORD0, 
	out float4 outPos: POSITION, 
	out float2 outTex: TEXCOORD0, 
	out float3 outNormal: TEXCOORD1, 
	out float3 outViewDir: TEXCOORD2) 
	{ 
		outPos = mul(inPos, matWorldViewProj); 
		outNormal = normalize(mul(inNormal, matWorld));; 
		outTex = inTex; 
		outViewDir = vecViewPos - mul(inPos, matWorld); 
	} 

	float4 PSC ( 
	in float2 inTex: TEXCOORD0, 
	in float3 inNormal: TEXCOORD1, 
	in float4 inViewDir: TEXCOORD2) : COLOR0
	{ 
		inNormal = normalize(inNormal); 
		float4 diffuse = 0.4 * saturate(10 * dot(-vecSunDir, inNormal)); 
		float4 colour = tex2D(ColourSampler, inTex); 
		float3 R = normalize(2 * dot(inNormal, -vecSunDir) * inNormal + vecSunDir); 
		float specular = pow(saturate(dot(R, normalize(inViewDir))), 100) * 0.2;
		colour.a *= fAlpha;
		return (diffuse + float4(0.4, 0.4, 0.4, 1.0) + specular) * colour;
	} 

	// LINES
	void VSL	(in float4 inPos		:	POSITION,
	in float2 inTex		:	TEXCOORD0,
	in float4 inNormal	:	TEXCOORD1,
	out float4 outPos		:	POSITION,
	out float2 outTex		:	TEXCOORD0) {
		inPos -= inNormal * 0.05;
		outPos = mul(inPos, matWorldViewProj);
		outTex = inTex;
	}

	float4 PSL	(in float2 inTex	:	TEXCOORD0) : COLOR0 {
		return tex2D(ColourSampler, inTex) * 0.2;
	}

	technique toon {
		pass depthSet {
			ZWriteEnable = TRUE;
			CullMode = CW;
			AlphaBlendEnable = TRUE;
			ZFunc = LESS;
			DestBlend = ONE;
			SrcBlend = ZERO;
			VertexShader = compile vs_2_0 VSD();
			PixelShader = compile ps_2_0 PSD();
		}
		pass colours {
			ZWriteEnable = TRUE;
			CullMode = CCW;
			AlphaBlendEnable = TRUE;
			ZFunc = LESSEQUAL;
			DestBlend = INVSRCALPHA;
			SrcBlend = SRCALPHA;
			VertexShader = compile vs_2_0 VSC();
			PixelShader = compile ps_2_0 PSC();
		}
		pass outLines {
			ZWriteEnable = TRUE;
			CullMode = CW;
			ZFunc = LESS;
			AlphaBlendEnable = FALSE;
			VertexShader = compile vs_2_0 VSL();
			PixelShader = compile ps_2_0 PSL();
		}
	}
	";
}

MATERIAL* mtl_toonBlack = {
	effect = "
	const float4x4 matWorldViewProj;
	const float4x4 matWorld;
	const float4 vecViewPos;
	const float4 vecSunDir;
	float fAlpha;

	// Actual Texture
	texture entSkin1;
	sampler ColourSampler = sampler_state {
		Texture = <entSkin1>;
		MipFilter = Linear;
	};

	// DEPTH
	void VSD	(in float4 inPos		:	POSITION,
	out float4 outPos		:	POSITION) {
		outPos = mul(inPos, matWorldViewProj);
	}

	float4 PSD	() : COLOR0 {
		return (1.0, 1.0, 1.0, 1.0);
	}

	// COLOURS
	void VSC ( 
	in float4 inPos: POSITION, 
	in float3 inNormal: NORMAL, 
	in float2 inTex: TEXCOORD0, 
	out float4 outPos: POSITION, 
	out float2 outTex: TEXCOORD0, 
	out float3 outNormal: TEXCOORD1, 
	out float3 outViewDir: TEXCOORD2) 
	{ 
		outPos = mul(inPos, matWorldViewProj); 
		outNormal = normalize(mul(inNormal, matWorld));; 
		outTex = inTex; 
		outViewDir = vecViewPos - mul(inPos, matWorld); 
	} 

	float4 PSC ( 
	in float2 inTex: TEXCOORD0, 
	in float3 inNormal: TEXCOORD1, 
	in float4 inViewDir: TEXCOORD2) : COLOR0
	{ 
		inNormal = normalize(inNormal); 
		float4 diffuse = 0.4 * saturate(10 * dot(-vecSunDir, inNormal)); 
		float4 colour = tex2D(ColourSampler, inTex); 
		float3 R = normalize(2 * dot(inNormal, -vecSunDir) * inNormal + vecSunDir); 
		float specular = pow(saturate(dot(R, normalize(inViewDir))), 100) * 0.2;
		colour.a *= fAlpha;
		return (diffuse + float4(0.4, 0.4, 0.4, 1.0) + specular) * colour;
	} 

	// LINES
	void VSL	(in float4 inPos		:	POSITION,
	in float2 inTex		:	TEXCOORD0,
	in float4 inNormal	:	TEXCOORD1,
	out float4 outPos		:	POSITION,
	out float2 outTex		:	TEXCOORD0) {
		inPos -= inNormal * 0.05;
		outPos = mul(inPos, matWorldViewProj);
		outTex = inTex;
	}

	float4 PSL	(in float2 inTex	:	TEXCOORD0) : COLOR0 {
		return float4(0.0, 0.0, 0.0, 1.0);
	}

	technique toon {
		pass depthSet {
			ZWriteEnable = TRUE;
			CullMode = CW;
			AlphaBlendEnable = TRUE;
			ZFunc = LESS;
			DestBlend = ONE;
			SrcBlend = ZERO;
			VertexShader = compile vs_2_0 VSD();
			PixelShader = compile ps_2_0 PSD();
		}
		pass colours {
			ZWriteEnable = TRUE;
			CullMode = CCW;
			AlphaBlendEnable = TRUE;
			ZFunc = LESSEQUAL;
			DestBlend = INVSRCALPHA;
			SrcBlend = SRCALPHA;
			VertexShader = compile vs_2_0 VSC();
			PixelShader = compile ps_2_0 PSC();
		}
		pass outLines {
			ZWriteEnable = TRUE;
			CullMode = CW;
			ZFunc = LESS;
			AlphaBlendEnable = FALSE;
			VertexShader = compile vs_2_0 VSL();
			PixelShader = compile ps_2_0 PSL();
		}
	}
	";
}
