// softRays10.fx
// Julian "Jibb" Smart
// 2009

const float4x4 matWorldViewProj;
const float4x4 matViewProj;

float toneMappingFactor_var;

// Origin on-screen
float4 vecSkill1;
float4 vecSkill5;
float4 vecSkill9;

float fAlpha;
// Convert clip space to texture space
static const float4x4 mat2Tex = {
	0.5f, 0.f, 0.f, 0.f,
	0.f, -0.5f, 0.f, 0.f,
	0.f, 0.f, 1.0f, 0.f,
	0.5f, 0.5f, 0.f, 1.0f
};

// Actual Texture
texture entSkin1;
sampler ColourMapSampler = sampler_state {
	Texture = <entSkin1>;
	MipFilter = Linear;
};

// Depth Map
texture depthMap_bmap;
sampler DepthSampler = sampler_state {
	Texture = <depthMap_bmap>;
	AddressU = Clamp;
	AddressV = Clamp;
	MipFilter = Linear;
};

// Noise for Jitter
texture noise_bmap;
sampler NoiseSampler = sampler_state {
	Texture = <noise_bmap>;
	AddressU = Wrap;
	AddressV = Wrap;
};

// VS
void VS	(in float4 inPos		:	POSITION,
			in float2 inTex		:	TEXCOORD0,
			out float4 outPos		:	POSITION,
			out float2 outTex		:	TEXCOORD0,
			out float4 depthTex	:	TEXCOORD1,
			out float4 origin		:	TEXCOORD2) {
	float4x4 matOffset = mul(matWorldViewProj, mat2Tex);
	float4 scaledPos = inPos;
	outPos = mul(inPos, matWorldViewProj);
	// Get the position we want to check
	depthTex = mul(inPos, matOffset);
	matOffset = mul(matViewProj, mat2Tex);
	origin = mul(vecSkill1, matOffset);

	outTex = inTex;
}

float4 PS	(in float2 inTex		:	TEXCOORD0,
				in float4 depthTex	:	TEXCOORD1,
				in float4 inPos		:	TEXCOORD2) : COLOR0 {
	float4 colour = tex2D(ColourMapSampler, inTex);
	float4 lookUp = (inPos - depthTex);
	float finalOcclusion = 0.0;
	float occlusion;
	float4 noise = tex2D(NoiseSampler, inTex * 100);
	float scale = noise.r * 0.4 + 0.6;
	for (int i = 0; i < 8; i++) {
		occlusion = tex2Dproj(DepthSampler, depthTex + lookUp * scale).r;
		finalOcclusion += 0.13 * saturate(0.05 * (vecSkill5.w - occlusion));
		scale *= 0.75;
	}
	// fade out if too close to the screen:
	colour.a *= fAlpha*saturate(0.01*inPos.z)*saturate(1.0-finalOcclusion)*saturate(toneMappingFactor_var)*vecSkill9.x;
	colour.rgb *= vecSkill5.xyz;
	return colour;
}

technique softRays3 {
	pass p0 {
		AlphaBlendEnable = TRUE;
		ZWriteEnable = FALSE;
		ZFunc = ALWAYS;
		DestBlend = ONE;
		SrcBlend = SRCALPHA;
		VertexShader = compile vs_3_0 VS();
		PixelShader = compile ps_3_0 PS();
	}
}

// fallback
technique softRays2 {
	pass p0 {
		AlphaBlendEnable = TRUE;
		ZWriteEnable = FALSE;
		ZFunc = ALWAYS;
		DestBlend = ONE;
		SrcBlend = SRCALPHA;
		VertexShader = compile vs_2_0 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
